// NotesMailDB.cpp: implementation of the CNotesMailDB class.
//
//////////////////////////////////////////////////////////////////////

/*******************************************************************************
 *                                                                             *
 *  This file is part of NotesAntiSpam.                                        *
 *                                                                             *
 *  NotesAntiSpam is free software; you can redistribute it and/or modify      *
 *  it under the terms of the GNU General Public License as published by       *
 *  the Free Software Foundation; either version 2 of the License, or          *
 *  (at your option) any later version.                                        *
 *                                                                             *
 *  NotesAntiSpam is distributed in the hope that it will be useful,           *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              *
 *  GNU General Public License for more details.                               *
 *                                                                             *
 *  You should have received a copy of the GNU General Public License          *
 *  along with NotesAntiSpam; if not, write to the Free Software               *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  *
 *                                                                             *
 *  Copyright (c) 2003 Thomas Kriener                                          *
 *                                                                             *
 *******************************************************************************/

#include "stdafx.h"
#include <Winsock2.h>
#include "NotesAntiSpam.h"
#include "NotesAntiSpamDlg.h"
#include "NotesMailDB.h"
#include "NotesMail.h"
#include "Debugger.h"
#include "WhitelistEntry.h"
#include "TrainIDCache.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define ERR_BUF_SIZE 2048

/*
 * Our Bayesian Filter
 */
#ifdef SPAMBAYES
CSpamBayes CNotesMailDB::m_bayes=CSpamBayes();
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CNotesMailDB::CNotesMailDB()
{
	CString tmp;

	// Get Pointer to Application
	myApp=(CNotesAntiSpamApp*)AfxGetApp();

	// copy originial notes.ini to notesantispam.ini
	CString configFile=CopyIniFile();

	// Make the error handler throw all errors encountered during execution.
	LNSetThrowAllErrors (TRUE);

	try
	{
		// Initialize the API.
		myApp->debugger.Add("Initializing Notes-Session");
		m_notesSession.Init(configFile);
		if(myApp->m_config.m_mailBox=="")
		{
			// get user's default mail database.
			myApp->debugger.Add("Get Default-Mailbox");
			m_notesSession.GetMailDatabase(&m_mailDb);
		}
		else
		{
			// open specified mail-database
			tmp.Format("Get specific Mailbox %s on server %s",myApp->m_config.m_mailBox,myApp->m_config.m_mailBoxServer);
			myApp->debugger.Add(tmp);
			LNString mailBox=myApp->m_config.m_mailBox;
			LNString mailBoxServer=myApp->m_config.m_mailBoxServer;
			m_notesSession.GetDatabase(mailBox, &m_mailDb, mailBoxServer);
		}
		// Open the database.
		myApp->debugger.Add("Opening Mailbox");
		m_mailDb.Open();
		// Open Inbox-View
		myApp->debugger.Add("Opening Inbox");
		m_mailDb.GetViewFolder("($Inbox)",&m_inboxView);
		m_inboxView.Open(LNVFOPENFLAGS_USE_UNREAD_LIST);
	}
	catch (LNSTATUS lnerror)
	{
		char ErrorBuf[ERR_BUF_SIZE];

		LNGetErrorMessage(lnerror, ErrorBuf, ERR_BUF_SIZE);
		tmp.Format("Notes-Error(%i): %s",lnerror,ErrorBuf);
		myApp->debugger.Add("**** ERROR: "+tmp);
		MessageBox(AfxGetMainWnd()->m_hWnd,tmp,"NotesAntiSpam",MB_OK|MB_ICONSTOP);
		return;
	}
	// Make the error handler throw no errors encountered during execution.
	// We know that the Folder "-Spam" might not be there
	LNSetThrowAllErrors (FALSE);
	// Open or create Spam-Folder
	if(m_mailDb.GetViewFolder("-Spam",&m_spamView)!=LNNOERROR)
	{
		if(m_mailDb.CreateFolder("-Spam")!=LNNOERROR)
		{
			myApp->debugger.Add("ERROR: Unable to create Folder \"-Spam\" !");
			MessageBox(AfxGetMainWnd()->m_hWnd,"Unable to create Folder \"-Spam\" !",
			           "NotesAntiSpam",MB_OK|MB_ICONSTOP);
			return;
		}
		if(m_mailDb.GetViewFolder("-Spam",&m_spamView)!=LNNOERROR)
		{
			myApp->debugger.Add("ERROR: Unable to open Folder \"-Spam\" !");
			MessageBox(AfxGetMainWnd()->m_hWnd,"Unable to open Folder \"-Spam\" after creation!",
			           "NotesAntiSpam",MB_OK|MB_ICONSTOP);
			return;
		}
	}
#ifdef SPAMBAYES
	// Open or create Spam-Folder for Training
	if(m_mailDb.GetViewFolder("-Spam\\TrainSpam",&m_spamTrainView)!=LNNOERROR)
	{
		if(m_mailDb.CreateFolder("-Spam\\TrainSpam")!=LNNOERROR)
		{
			myApp->debugger.Add("ERROR: Unable to create Folder \"-Spam\\TrainSpam\" !");
			MessageBox(AfxGetMainWnd()->m_hWnd,"Unable to create Folder \"-Spam\\TrainSpam\" !",
			           "NotesAntiSpam",MB_OK|MB_ICONSTOP);
			return;
		}
		if(m_mailDb.GetViewFolder("-Spam\\TrainSpam",&m_spamTrainView)!=LNNOERROR)
		{
			myApp->debugger.Add("ERROR: Unable to open Folder \"-Spam\\TrainSpam\" !");
			MessageBox(AfxGetMainWnd()->m_hWnd,"Unable to open Folder \"-Spam\\TrainSpam\" after creation!",
			           "NotesAntiSpam",MB_OK|MB_ICONSTOP);
			return;
		}
	}
	// Open or create Spam-Folder for Training
	if(m_mailDb.GetViewFolder("-Spam\\TrainHam",&m_spamTrainView)!=LNNOERROR)
	{
		if(m_mailDb.CreateFolder("-Spam\\TrainHam")!=LNNOERROR)
		{
			myApp->debugger.Add("ERROR: Unable to create Folder \"-Spam\\TrainHam\" !");
			MessageBox(AfxGetMainWnd()->m_hWnd,"Unable to create Folder \"-Spam\\TrainHam\" !",
			           "NotesAntiSpam",MB_OK|MB_ICONSTOP);
			return;
		}
		if(m_mailDb.GetViewFolder("-Spam\\TrainHam",&m_spamTrainView)!=LNNOERROR)
		{
			myApp->debugger.Add("ERROR: Unable to open Folder \"-Spam\\TrainHam\" !");
			MessageBox(AfxGetMainWnd()->m_hWnd,"Unable to open Folder \"-Spam\\TrainHam\" after creation!",
			           "NotesAntiSpam",MB_OK|MB_ICONSTOP);
			return;
		}
	}
#endif
	// Make the error handler throw all errors encountered during execution.
	LNSetThrowAllErrors (TRUE);
	try
	{
		myApp->debugger.Add("Opening SpamView");
		m_spamView.Open();
#ifdef SPAMBAYES
		myApp->debugger.Add("Opening SpamTrainView");
		m_spamTrainView.Open();
		myApp->debugger.Add("Opening HamTrainView");
		m_hamTrainView.Open();
#endif
	}
	catch (LNSTATUS lnerror)
	{
		char ErrorBuf[ERR_BUF_SIZE];

		LNGetErrorMessage(lnerror, ErrorBuf, ERR_BUF_SIZE);
		tmp.Format("Notes-Error on Spam-Folder (%i): %s",lnerror,ErrorBuf);
		myApp->debugger.Add("**** ERROR: "+tmp);
		MessageBox(AfxGetMainWnd()->m_hWnd,tmp,"NotesAntiSpam",MB_OK|MB_ICONSTOP);
		return;
	}
}

CNotesMailDB::~CNotesMailDB()
{
	CString tmp;

	// Now for the main processing.
	// Make the error handler throw all errors encountered during execution.
	LNSetThrowAllErrors (TRUE);

	try
	{
#ifdef SPAMBAYES
		// Close TrainSpam-View
		if(m_spamTrainView.IsOpen())
			m_spamTrainView.Close();
#endif
		// Close the Spam-View
		m_spamView.Close();
		// Close the Inbox-View
		m_inboxView.Close();
		// Close the database.
		m_mailDb.Close();
		// Terminate the API.
		m_notesSession.Term();
	}
	catch (LNSTATUS lnerror)
	{
		char ErrorBuf[ERR_BUF_SIZE];

		LNGetErrorMessage(lnerror, ErrorBuf, ERR_BUF_SIZE);
		tmp.Format("Notes-Error(%i): %s",lnerror,ErrorBuf);
		myApp->debugger.Add("**** ERROR: "+tmp);
		MessageBox(AfxGetMainWnd()->m_hWnd,tmp,"NotesAntiSpam",MB_OK|MB_ICONSTOP);
		return;
	}
}

/*
 * searchSpam searches for Spam in the current Mailbox
 *
 * returns TRUE on normal exit
 *         FALSE if interrupted by m_hEventKillSearchThread
 */
BOOL CNotesMailDB::searchSpam(CDebugger *debugger, CSearchThreadInfo* pSearchInfo)
{
	LNINT			index;
	LNINT			numDocs;
	CString         tmp;
	BOOL            noSpam=FALSE;

	try
	{
		LNItemArray itemArray;

		debugger->Add("**************************************************************************************");
		debugger->Add("*  NEW RUN                                                                           *");
		debugger->Add("**************************************************************************************");
		((CNotesAntiSpamDlg*)AfxGetMainWnd())->AddLog("\r\n");
#ifdef _DEBUG
		tmp="----New Run----\r\n";
		((CNotesAntiSpamDlg*)AfxGetMainWnd())->AddLog(tmp);
#endif

		// Refresh View "Inbox"
		m_inboxView.Refresh();

		numDocs = m_inboxView.GetEntryCount();
		tmp.Format("Document Count:  %i", numDocs);
		debugger->Add(tmp);
		
		for (index = 0; index < numDocs; index++)
		{
			//LNMailMessage curDoc;
			CNotesMail  curDoc;
			LNVFEntry   entry;  // Working entry
			LNText      textItem;
			LNString    tmpLNStr;
			CString     sender,subject;
			CString     dnsbl;
			BOOL        spam=FALSE;
			BOOL        whitelist=FALSE;
			CString     messageID;


			// Check if Thread should be killed
			if (WaitForSingleObject(pSearchInfo->m_hEventKillSearchThread, 0)  == WAIT_OBJECT_0)
			{
				/*
				if (hwndNotifyProgress != NULL)
				{
					// Reset progress indication to 0 (recalculation not begun)
					pWndNotifyProgress->PostMessage(WM_USER_RECALC_IN_PROGRESS);
				}
				*/
				return FALSE; // Terminate this recalculation
			}


			// Get Entry
			entry=m_inboxView[index];
			// we check only unread documents
			if(entry.IsUnread())
			{
				/*
				 * Check document
				 */
				entry.GetDocument(&curDoc);

				// open MailDocument
				curDoc.Open();

				// Check if Document already processed
				messageID=curDoc.GetMessageID();
				if(m_noteIDCache.Check(messageID))
				{
					if(debugger->GetState())
					{
						tmp.Format("**** MessageID %s already scanned ****\r\n",messageID);
						debugger->Add(tmp);
						((CNotesAntiSpamDlg*)AfxGetMainWnd())->AddLog(tmp);
					}
					else
						((CNotesAntiSpamDlg*)AfxGetMainWnd())->AddLog(".");
					curDoc.Close();
					continue;
				}
				else
				{
					m_noteIDCache.AddEntry(messageID);
					if(debugger->GetState())
					{
						tmp.Format("**** scanning MessageID %s ****\r\n",messageID);
						debugger->Add(tmp);
						((CNotesAntiSpamDlg*)AfxGetMainWnd())->AddLog(tmp);
					}
					else
						((CNotesAntiSpamDlg*)AfxGetMainWnd())->AddLog("+");
				}


				// Write the mail message to the log file if debug active.
				if(debugger->GetState())
				{
					debugger->Add("*******************************************************************************************");
					debugger->Add(curDoc.dump());
					debugger->Add("*******************************************************************************************");
				}

				// Get Subject for log
				subject=curDoc.GetSubjectString();

				// get Sender
				curDoc.GetSender(&tmpLNStr);
				sender=tmpLNStr.GetPlatformTextPtr();
				// check against whitelist
				if(checkAgainstWhitelist(sender))
				{
					tmp.Format("########### Sender %s listed in Whitelist ###########",sender);
					debugger->Add(tmp);
					whitelist=TRUE;
				}

#ifdef SPAMBAYES
				// Check with Bayes-Filter
				if(myApp->m_config.m_bayesFilter && (!whitelist))
				{
					if(m_bayes.CheckSpam(curDoc.dump()))
					{
						dnsbl="Bayes";
						spam=true;
						tmp.Format("########### Bayesian Filter found spam from %s ###########",sender);
						debugger->Add(tmp);
					}
				}
#endif
				// Check with DNSBL
				if(myApp->m_config.m_dnsblFilter && (!whitelist))
				{
					// Get Received lines for passed Mailgateways
					curDoc.GetReceivedLines(&textItem);
					for (unsigned int i = 0; i < textItem.GetCount(); i++)
					{
						CIPAddress ipAddress;	

						// Stop if Spam already identified
						if(spam) break;

						// get line
						tmpLNStr=textItem[i];
						tmp.Format("Received: %s",tmpLNStr.GetPlatformTextPtr());
						debugger->Add(tmp);

						// extract IP-Address
						ipAddress=extractIPAddress(tmpLNStr);

						tmp.Format("extracted IP-Address: %s",ipAddress.toString());
						debugger->Add(tmp);

						// check against good Cache
						if(m_goodDnsCache.Check(ipAddress))
						{
#ifdef _DEBUG
							tmp.Format("########### IP-Address %s listed in good Cache ###########",ipAddress.toString());
							debugger->Add(tmp);
#endif
							continue;
						}
						// check against bad Cache
						else if(m_dnsblCache.Check(ipAddress))
						{
							spam=TRUE;
							dnsbl=m_dnsblCache.GetDnsbl4IPAddress(ipAddress);
							tmp.Format("########### IP-Address %s listed in Cache by %s ###########",ipAddress.toString(),dnsbl);
							debugger->Add(tmp);
						}
						// check against DNSBL
						else if(checkAgainstDnsbl(ipAddress,dnsbl))
						{
							tmp.Format("########### IP-Address %s listed by %s ###########",ipAddress.toString(),dnsbl);
							debugger->Add(tmp);
							spam=TRUE;
							m_dnsblCache.AddEntry(ipAddress,dnsbl);
						}
						// IP-Address not in DNS Blacklists
						else
						{
							m_goodDnsCache.AddEntry(ipAddress);
						}
					}
				}

				// Close Document
				curDoc.Close();

				// if spam found move document from "Inbox" to "-Spam"
				if(spam&&(!whitelist))
				{
					// create log
					tmp.Format("\r\n%s: spam from \"%s\" with subject \"%s\"\r\n",dnsbl,sender,subject);
					((CNotesAntiSpamDlg*)AfxGetMainWnd())->AddLog(tmp);

					// change subject
					tmp.Format("[SPAM:%s] %s",dnsbl,subject);
					curDoc.Open();
					curDoc.SetSubjectString(tmp);
					curDoc.Save();
					curDoc.Close();

					// move document
					if(m_spamView.FolderAddDocument(curDoc)==LNNOERROR)
						m_inboxView.FolderRemoveDocument(curDoc);
				}
				else
				{
					noSpam=TRUE;
				}
				if(whitelist)
				{
					tmp.Format("\r\nMail from whitelist-sender \"%s\" with subject \"%s\"\r\n",sender,subject);
					((CNotesAntiSpamDlg*)AfxGetMainWnd())->AddLog(tmp);
				}
			}
		}

	}
	catch (LNSTATUS lnerror)
	{
		if(lnerror==3221225501)
		{
			((CNotesAntiSpamDlg*)AfxGetMainWnd())->AddLog("IGNORED: Array Subscript out of Rage\r\n");
			return TRUE;
		}
		else
		{
			char ErrorBuf[ERR_BUF_SIZE];
			
			LNGetErrorMessage(lnerror, ErrorBuf, ERR_BUF_SIZE);
			tmp.Format("Notes-Error(%u): %s",lnerror,ErrorBuf);
			myApp->debugger.Add("**** ERROR: "+tmp);
			MessageBox(AfxGetMainWnd()->m_hWnd,tmp,"NotesAntiSpam",MB_OK|MB_ICONSTOP);
			return TRUE;
		}
	}

	if(noSpam && myApp->m_config.m_informNewMail)
	{
		AfxGetMainWnd()->PostMessage(MYWM_INFORMNEWMAIL);
	}

	return TRUE;
}

CIPAddress CNotesMailDB::extractIPAddress(const LNString &lnStr)
{
	CString    str,tmp1,tmp2;

	str=lnStr;
	tmp1=str.Right(str.GetLength()-str.Find('[')-1);
	tmp2=tmp1.Left(tmp1.Find(']'));

	return CIPAddress(tmp2);
}

BOOL CNotesMailDB::checkAgainstDnsbl(const CIPAddress &ip, CString &dnsbl)
{
	CString tmp;
	hostent* myHost;

	for(int i=0; i<myApp->m_config.m_dnsbl.GetSize(); i++)
	{
		tmp=ip.reverseIPString()+myApp->m_config.m_dnsbl[i].m_domain;
		myHost=gethostbyname(tmp);
		if(myHost!=NULL)
		{
			dnsbl=myApp->m_config.m_dnsbl[i].m_name;
			return TRUE;
		}
	}
	return FALSE;
}

BOOL CNotesMailDB::checkAgainstWhitelist(const CString &sender)
{
	CWhitelistEntry entry;
	for(int i=0; i<myApp->m_config.m_whitelist.GetSize(); i++)
	{
		entry=myApp->m_config.m_whitelist[i];
		if(sender.Find(entry.m_address)>=0)
			return TRUE;
	}
	return FALSE;
}

CString CNotesMailDB::CopyIniFile()
{
	CString strFrom = "notes.ini";
	CString strTo   = "NotesAntiSpam.ini";
//	CString notesPath;
	char szBuffer[1024];
	CString tmp;

/*	// Read the Lotus-Notes Directory
	if(0>=GetPrivateProfileString("Lotus Applications",
	                              "Notes",
								  "",
								  szBuffer,
								  sizeof(szBuffer)-1,
								  "lotus.ini"))
	{
		MessageBox(NULL,"wrong lotus.ini","NotesAntiSpam",MB_OK|MB_ICONERROR);
		return "";
	}
	notesPath=szBuffer;
	notesPath+="\\";*/

	strFrom=myApp->m_config.m_notesIniFilename;
	strTo=myApp->m_config.m_userpath+strTo;

	// copy original file
	if(!CopyFile(strFrom,strTo,FALSE))
	{
		tmp.Format("***** ERROR: Unable to copy notes.ini from %s to %s",strFrom,strTo);
		myApp->debugger.Add(tmp);
	}

	// Read Addins
	GetPrivateProfileString("Notes",
	                        "EXTMGR_ADDINS",
							"",
							szBuffer,
							sizeof(szBuffer)-1,
							strTo);

	tmp=szBuffer;
	if(tmp.GetLength()>0)
	{
		if(tmp.Find("NASExtPwd")<0)
			tmp+=",NASExtPwd";
	}
	else
	{
		tmp="NASExtPwd";
	}

	WritePrivateProfileString("Notes",
	                          "EXTMGR_ADDINS",
							  tmp,
							  strTo);

	return strTo;
}
/*
 * trainBayes: trains the bayes filter based on Inbox and -Spam
 *
 * returns TRUE on normal exit
 *         FALSE if interrupted by m_hEventKillSearchThread
 */
#ifdef SPAMBAYES
BOOL CNotesMailDB::trainBayes(CDebugger *debugger, CSearchThreadInfo* pSearchInfo)
{
	LNINT			index;
	LNINT			numDocs;
	CString         tmp;
	CTrainIDCache	spamCache("idspam.gdbm");
	CTrainIDCache   hamCache("idham.gdbm");

	try
	{
		LNItemArray itemArray;

		debugger->Add("**************************************************************************************");
		debugger->Add("*  NEW TRAINBAYES RUN                                                                           *");
		debugger->Add("**************************************************************************************");
		((CNotesAntiSpamDlg*)AfxGetMainWnd())->AddLog("\r\n");
#ifdef _DEBUG
		tmp="----New TrainBayes-Run----\r\n";
		((CNotesAntiSpamDlg*)AfxGetMainWnd())->AddLog(tmp);
#endif

		debugger->Add("TRAIN HAM FROM -Spam\\TrainHam");
		((CNotesAntiSpamDlg*)AfxGetMainWnd())->AddLog("Train Ham from Inbox\r\n");

		// Refresh View "Inbox"
		m_hamTrainView.Refresh();

		numDocs = m_hamTrainView.GetEntryCount();
		tmp.Format("Document Count:  %i", numDocs);
		debugger->Add(tmp);
		
		for (index = 0; index < numDocs; index++)
		{
			CNotesMail  curDoc;
			LNVFEntry   entry;  // Working entry
			LNText      textItem;
			LNString    tmpLNStr;
			NOTEID      noteID;


			// Check if Thread should be killed
			if (WaitForSingleObject(pSearchInfo->m_hEventKillSearchThread, 0)  == WAIT_OBJECT_0)
			{
				/*
				if (hwndNotifyProgress != NULL)
				{
					// Reset progress indication to 0 (recalculation not begun)
					pWndNotifyProgress->PostMessage(WM_USER_RECALC_IN_PROGRESS);
				}
				*/
				return FALSE; // Terminate this recalculation
			}


			// Get Entry
			entry=m_hamTrainView[index];
			noteID=entry.GetNoteID();

			if(hamCache.Check(noteID))
			{
				if(debugger->GetState())
				{
					tmp.Format("**** NoteID %i already trained ****",noteID);
					debugger->Add(tmp);
#ifdef _DEBUG
					tmp+="\r\n";
					((CNotesAntiSpamDlg*)AfxGetMainWnd())->AddLog(tmp);
#else
					((CNotesAntiSpamDlg*)AfxGetMainWnd())->AddLog(".");
#endif
				}
				continue;
			}
			else
			{
				hamCache.AddEntry(noteID);
				if(debugger->GetState())
				{
					tmp.Format("**** training NoteID %i ****\r\n",noteID);
#ifdef _DEBUG
					((CNotesAntiSpamDlg*)AfxGetMainWnd())->AddLog(tmp);
#else
					((CNotesAntiSpamDlg*)AfxGetMainWnd())->AddLog("+");
#endif
				}
			}
			/*
			 * Check document
			 */
			entry.GetDocument(&curDoc);

			// Write the mail message to the log file if debug active.
			if(debugger->GetState())
			{
				debugger->Add("*******************************************************************************************");
				debugger->Add(curDoc.dump());
				debugger->Add("*******************************************************************************************");
			}

			// open MailDocument
			curDoc.Open();

			// train as ham
			m_bayes.TrainHam(curDoc.dump());

			// close MailDocument
			curDoc.Open();

			// move to "Inbox" after trained
			if(m_inboxView.FolderAddDocument(curDoc)==LNNOERROR)
				m_hamTrainView.FolderRemoveDocument(curDoc);
		}

		debugger->Add("*******************************************************************************************");
		debugger->Add("TRAIN SPAM FROM -Spam\SpamTrain");
		((CNotesAntiSpamDlg*)AfxGetMainWnd())->AddLog("Train Spam from -Spam\\SpamTrain\r\n");

		// Refresh View "-Spam\\SpamTrain"
		m_spamTrainView.Refresh();

		numDocs = m_spamTrainView.GetEntryCount();
		tmp.Format("Document Count:  %i", numDocs);
		debugger->Add(tmp);
		
		for (index = 0; index < numDocs; index++)
		{
			CNotesMail  curDoc;
			LNVFEntry   entry;  // Working entry
			LNText      textItem;
			LNString    tmpLNStr;
			NOTEID      noteID;


			// Check if Thread should be killed
			if (WaitForSingleObject(pSearchInfo->m_hEventKillSearchThread, 0)  == WAIT_OBJECT_0)
			{
				/*
				if (hwndNotifyProgress != NULL)
				{
					// Reset progress indication to 0 (recalculation not begun)
					pWndNotifyProgress->PostMessage(WM_USER_RECALC_IN_PROGRESS);
				}
				*/
				return FALSE; // Terminate this recalculation
			}


			// Get Entry
			entry=m_spamTrainView[index];
			noteID=entry.GetNoteID();

			if(spamCache.Check(noteID))
			{
				if(debugger->GetState())
				{
					tmp.Format("**** NoteID %i already trained ****",noteID);
					debugger->Add(tmp);
#ifdef _DEBUG
					tmp+="\r\n";
					((CNotesAntiSpamDlg*)AfxGetMainWnd())->AddLog(tmp);
#else
					((CNotesAntiSpamDlg*)AfxGetMainWnd())->AddLog(".");
#endif
				}
				continue;
			}
			else
			{
				spamCache.AddEntry(noteID);
				if(debugger->GetState())
				{
					tmp.Format("**** training NoteID %i ****\r\n",noteID);
#ifdef _DEBUG
					((CNotesAntiSpamDlg*)AfxGetMainWnd())->AddLog(tmp);
#else
					((CNotesAntiSpamDlg*)AfxGetMainWnd())->AddLog("+");
#endif
				}
			}

			/*
			 * Check document
			 */
			entry.GetDocument(&curDoc);

			// Write the mail message to the log file if debug active.
			if(debugger->GetState())
			{
				debugger->Add("*******************************************************************************************");
				debugger->Add(curDoc.dump());
				debugger->Add("*******************************************************************************************");
			}

			// open MailDocument
			curDoc.Open();

			// train as Spam
			m_bayes.TrainSpam(curDoc.dump());

			// close MailDocument
			curDoc.Open();
			
			// move to "-Spam" after trained
			if(m_spamView.FolderAddDocument(curDoc)==LNNOERROR)
				m_spamTrainView.FolderRemoveDocument(curDoc);
		}
	}
	catch (LNSTATUS lnerror)
	{
		char ErrorBuf[ERR_BUF_SIZE];

		LNGetErrorMessage(lnerror, ErrorBuf, ERR_BUF_SIZE);
		tmp.Format("Notes-Error(%i): %s",lnerror,ErrorBuf);
		myApp->debugger.Add("**** ERROR: "+tmp);
		MessageBox(AfxGetMainWnd()->m_hWnd,tmp,"NotesAntiSpam",MB_OK|MB_ICONSTOP);
		return TRUE;
	}

	return TRUE;
}
#endif

LNDatetime CNotesMailDB::GetLastModified()
{
	LNDatetime lastMod;
	CString    tmp;

	lastMod.SetDate(1,1,51);
	lastMod.SetTime(0,0,0,0);

	try
	{
		lastMod=m_mailDb.GetLastModified();
	}
	catch (LNSTATUS lnerror)
	{
		char ErrorBuf[ERR_BUF_SIZE];

		LNGetErrorMessage(lnerror, ErrorBuf, ERR_BUF_SIZE);
		tmp.Format("Notes-Error(%i): %s",lnerror,ErrorBuf);
		myApp->debugger.Add("**** ERROR: "+tmp);
		((CNotesAntiSpamDlg*)AfxGetMainWnd())->AddLog(tmp);
		return lastMod;
	}

	return lastMod;
}
